@@ -183,7 +183,7 @@ class TimeSignal(Signal):
 
     def power(self, dB=True):
         power = np.mean(self.samples ** 2)
-        return 10*np.log10(power) if dB else power
+        return 10*np.log10(max(power, 1e-12)) if dB else power
 
 
     def filter(self, coefficients):
@@ -225,6 +225,48 @@ class TimeSignal(Signal):
         return np.array([samples[i:i+frame_size] for i in range(0, len(samples)-frame_size+1, step_size)]).T
 
 
+    @classmethod
+    def from_processing_blocks(cls, input_signal, block_size, processing_fn, state_data=None, output_samplerate=None):
+        input_samples = input_signal.samples
+        total_length = len(input_signal)
+        output_samples = []
+
+        # Pass input sliced into blocks to processing function
+        for start in np.arange(0, total_length - block_size, block_size):
+            input_buffer = input_samples[start:start+block_size]
+            output_buffer = processing_fn(input_buffer, state_data)
+            if output_buffer is not None:
+                output_samples.append(output_buffer)
+
+        # Pass partial buffer at end of signal
+        start += block_size
+        final_output_buffer = processing_fn(input_samples[start:], state_data)
+        if final_output_buffer is not None:
+            output_samples.append(final_output_buffer)
+
+        if len(output_samples) > 0:
+            if output_samplerate is None:
+                output_samplerate = input_signal.samplerate
+            return cls(np.hstack(output_samples), output_samplerate)
+
+
+    @classmethod
+    def from_processing_recording(cls, recording_device, recording_samplerate, block_size, processing_fn, state_data=None, output_samplerate=None):
+        output_samples = []
+        with recording_device.recorder(samplerate=recording_samplerate, channels=1) as recorder:
+            try:
+                while True:
+                    recorded_buffer = np.squeeze(recorder.record(numframes=block_size))
+                    output_buffer = processing_fn(recorded_buffer, state_data)
+                    if output_buffer is not None:
+                        output_samples.append(output_buffer)
+            except KeyboardInterrupt:
+                if len(output_samples) > 0:
+                    if output_samplerate is None:
+                        output_samplerate = recording_samplerate
+                    return cls(np.hstack(output_samples), recording_samplerate)
+
+
 class AudioSignal(TimeSignal):
     """A class representing an audio signal with sample values between -1 and 1.
     
@@ -286,6 +328,18 @@ class AudioSignal(TimeSignal):
         return fig
 
 
+    @classmethod
+    def from_recording_device(cls, recording_device, recording_samplerate, block_size):
+        recorded_samples = []
+        with recording_device.recorder(samplerate=recording_samplerate, channels=1) as recorder:
+            try:
+                while True:
+                    recorded_buffer = np.squeeze(recorder.record(numframes=block_size))
+                    recorded_samples.append(recorded_buffer)
+            except KeyboardInterrupt:
+                return cls(np.hstack(recorded_samples), recording_samplerate)
+
+
 class Spectrum(Signal):
     """ A class representing a complex spectrum.
     """
@@ -333,7 +387,7 @@ class Spectrum(Signal):
             else:
                 self.magnitude[1:] *= sqrt(2)
         if dB:
-            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-6, None))
+            self.magnitude = 20 * np.log10(np.maximum(self.magnitude, 1e-6))
         else:
             self.magnitude **= exponent
 
@@ -398,7 +452,7 @@ class Spectrum(Signal):
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:-1]) + power_per_freq[-1]
         else:
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:])
-        return 10*np.log10(power) if dB else power
+        return 10*np.log10(max(power, 1e-12)) if dB else power
 
 
     def set_magnitude(self, value, start=None, end=None):
@@ -497,7 +551,7 @@ class Spectrogram(Signal):
             else:
                 self.magnitude[1:] *= sqrt(2)
         if dB:
-            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-6, None))
+            self.magnitude = 20 * np.log10(np.maximum(self.magnitude, 1e-6))
         else:
             self.magnitude **= exponent
 
@@ -573,7 +627,7 @@ class Spectrogram(Signal):
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:-1], axis=0) + power_per_freq[-1]
         else:
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:], axis=0)
-        return 10*np.log10(power) if dB else power
+        return 10*np.log10(np.maximum(power, 1e-12)) if dB else power
 
 
     @property
